//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.messaging.commands.*;

import java.util.*;
import java.math.BigDecimal;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.common.objects.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.user.objects.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.accesscontrol.*;
import com.ibm.commerce.messaging.util.*;
import com.ibm.commerce.pa.dbsync.*;
/**
 * The implementation of the ProductListPriceUpdate command (see interface for a detailed
 * description of the ProductListPriceUpdate command)
 */
public class ProductListPriceUpdateCmdImpl  extends ControllerCommandImpl implements ProductListPriceUpdateCmd
{
	/**
	 * The internal copyright field.
	 */	
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	private Long iCatalogEntryId = null;
	private String iProductSKUNumber = null;
	private String iCurrency = null;
	private BigDecimal iPrice = null;
	private Long iMemberId = null;
	/**
	 * A vector containing a request property for each update needed.
	 */
	public java.util.Vector iRequestPropertiesVector;
/**
 * Performs a verification check after the command executes.
 * @return boolean
 * @param aRequestProperty com.ibm.commerce.datatype.TypedProperty
 * @exception ECException
 */
public boolean doPostProcess(TypedProperty aRequestProperty) 
{
	return true;
}
/**
 * Performs a verification check before the command executes.
 * @return boolean
 * @param aRequestProperty com.ibm.commerce.datatype.TypedProperty
 * @exception ECException
 */
public boolean doPreProcess(TypedProperty aRequestProperty) 
{
	return true;
}
/**
 * Updates the product list price based on the request properties.  If a problem with
 * the update occurs, an ECException will be thrown.
 * @return boolean
 * @param aRequestProperty com.ibm.commerce.datatype.TypedProperty
 * @exception ECException
 */
public boolean doProcess(TypedProperty aRequestProperty) throws ECException 
{
	final String methodName = "doProcess";
	try {

		if (iCatalogEntryId == null) 
		{
			// Determine catentry_id from sku number/member Id combination
			CatalogEntryAccessBean foundCatEntryAB = new CatalogEntryAccessBean().findByMemberIdAndSKUNumber(iMemberId,iProductSKUNumber);
			iCatalogEntryId = foundCatEntryAB.getCatalogEntryReferenceNumberInEJBType();
		}
				  
		// Get the ListPrice access bean
		try {
			ListPriceAccessBean priceAB = new ListPriceAccessBean();
			priceAB.setInitKey_catalogEntryReferenceNumber(iCatalogEntryId.toString());
			priceAB.setInitKey_currency(iCurrency);
		
			//update price
			priceAB.setListPrice(iPrice);
			priceAB.commitCopyHelper();
		} catch (FinderException ex) {
			ListPriceAccessBean priceAB = new ListPriceAccessBean(iCatalogEntryId, iCurrency, iPrice);
			priceAB.commitCopyHelper();
		}

		// defect 7030
		// Support for MaintainSearchSpace feature (PCR144)
		if (com.ibm.commerce.pa.metaphor.ProductFamily.familiesExist()) {
			Vector productList = new Vector();
			productList.addElement(iCatalogEntryId);
			UpdateSearchSpaces uss = new UpdateSearchSpaces();
			uss.updateProducts(productList, null);
		}
	} 
	catch( javax.ejb.CreateException ex ) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName,
						ECMessageHelper.generateMsgParms( ex.getMessage()), ex );
	}
	catch( javax.naming.NamingException ex ) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch( java.rmi.RemoteException ex ) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, 
						ECMessageHelper.generateMsgParms(ex.getMessage()), ex );
	}
	catch (FinderException ex) {
		throw new ECApplicationException(ECMessage._ERR_PP_PRODUCT_NOT_FOUND, this.getClass().getName(), methodName);
	} 

	return true;
}
/**
 * This method returns the Organization unit reference numbers for all the resources accessed by this command
 * It will return null if no resource if being accessed by this command.
 *
 *	
 * @return Long[] - an array of all organization unit reference numbers
 */
public Long[] getResourceOwners() throws ECException {
	// get all resource owners for this command
	Long[] resowners = null;
	resowners = new Long[1];
	resowners[0] = new Long(ECConstants.EC_ACC_ALL_RESOURCES); // Access to all Owner's

	return resowners;
}
/**
 * Get resources.
 * Creation date: (8/30/2001 2:07:27 PM) 
 * @return com.ibm.commerce.accesscontrol.AccessVector
 * @exception com.ibm.commerce.exception.ECException The exception description.
 */
public AccessVector getResources() throws ECException {
/*	final String methodName = "getResources";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), methodName);
	com.ibm.commerce.catalog.objects.CatalogEntryAccessBean ceAB = null;
	
	try {
		ceAB = new com.ibm.commerce.catalog.objects.CatalogEntryAccessBean();
		ceAB.setInitKey_catalogEntryReferenceNumber(iCatalogEntryId.toString());
		ceAB.refreshCopyHelper();
		
	} catch (FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), methodName, ex);
	} catch (RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), methodName, ex);
	} catch (NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, this.getClass().getName(), methodName, ex);
	} catch (CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), methodName, ex);
	
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MESSAGING, this.getClass().getName(), methodName);
	return new AccessVector(ceAB);
*/
	return null;	
}
/**
 * Indicates whether the command is ready to execute.
 * @return boolean
 */
public boolean isReadyToCallExecute()
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "isReadyToCallExecute");

	if (!super.isReadyToCallExecute())
		return false;

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "isReadyToCallExecute");
	return true;
}
/**
 * Indicates whether each command instance is ready to be executed.
 * @return boolean
 */
public boolean isReadyToCallExecutePerCommandInstance()
{
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "isReadyToCallExecutePerCommandInstance");

	if (!super.isReadyToCallExecute())
		return false;

	if (iPrice == null) {
		ECMessageLog.out(ECMessage._ERR_DIDNT_SET_PRODUCT_PRICE, this.getClass().getName(), "isReadyToCallExecute");
		return false;
	}
	if (iCurrency == null) {
		ECMessageLog.out(ECMessage._ERR_DIDNT_SET_CURRENCY, this.getClass().getName(), "isReadyToCallExecute");
		return false;
	}
	
	if (iCatalogEntryId == null) 
	{
		if (iProductSKUNumber == null) 
		{
			ECMessageLog.out(ECMessage._ERR_DIDNT_SET_PRODUCT, this.getClass().getName(), "isReadyToCallExecute");
			return false;
		}
		if (iMemberId == null) 
		{
			ECMessageLog.out(ECMessage._ERR_PP_MEMBERID_NOT_SET, this.getClass().getName(), "isReadyToCallExecute");
			return false;
		}
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "isReadyToCallExecutePerCommandInstance");
	return true;
}
/**
 * Processes each request property in the iRequestPropertiesVector.  Before each
 * request property is processed, a pre condition check is performed.  If the
 * pre condition check on the request property is ok, the business logic will be
 * performed followed by a post condition check to ensure integrity is maintained. 
 * If either of the 3 steps fail, an ECException will be thrown.
 * @exception ECException
 */
public void performExecute() throws ECException
{
	final String methodName = "performExecute";
	
	super.performExecute();
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "performExecute");

	TypedProperty currRequestProperties=null;
	int numberOfCommands;
	boolean vector_fl = false;

	if (iRequestPropertiesVector == null) {
		currRequestProperties = requestProperties;
		numberOfCommands = 1;
	}
	else 
	{
		vector_fl = true;
		numberOfCommands = iRequestPropertiesVector.size();
	}

	for (int i=0; i< numberOfCommands; i++) 
	{
		if (vector_fl) 
		{
			currRequestProperties = (TypedProperty) iRequestPropertiesVector.elementAt(i);
		}
		setRequestPropertiesPerCommandInstance(currRequestProperties);
		if (!isReadyToCallExecutePerCommandInstance()) {
			throw new ECApplicationException(ECMessage._ERR_PP_PARAMS, this.getClass().getName(), methodName);
		} else 	{
			if (!doPreProcess(requestProperties)) {
				throw new ECApplicationException(ECMessage._ERR_DO_PRE_PROCESS, this.getClass().getName(), methodName);
			}	

			if (!doProcess(requestProperties)) {
				throw new ECApplicationException(ECMessage._ERR_DO_PROCESS, this.getClass().getName(), methodName);
			}	
				
			if (!doPostProcess(requestProperties)) {
				throw new ECApplicationException(ECMessage._ERR_DO_POST_PROCESS, this.getClass().getName(), methodName);
			}
		}
	}
		
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "performExecute");
}
/**
 * Sets the iRequestPropertiesVector based on the given aRequestProperties.
 * @param aRequestProperty com.ibm.commerce.datatype.TypedProperty
 */
 
public void setRequestProperties(TypedProperty aRequestProperties)
{
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "setRequestProperties");

		requestProperties = aRequestProperties;
		
		try 
		{
			iRequestPropertiesVector = (Vector) requestProperties.get(ICIConstants.ICI_TYPED_PROPERTY_VECTOR);
		} 
		catch (Exception ex) 
		{
			iRequestPropertiesVector = null;
		}	
				
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "setRequestProperties");
}
/**
 * Sets the command instance variables before processing each item in the
 * iRequestPropertiesVector.
 * @param aRequestProperty com.ibm.commerce.datatype.TypedProperty
 * @exception com.ibm.commerce.exception.ECApplicationException
 */
 
public void setRequestPropertiesPerCommandInstance(TypedProperty aRequestProperties) 
				throws com.ibm.commerce.exception.ECApplicationException
{
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "setRequestPropertiesPerCommandInstance");

		requestProperties = aRequestProperties;
		
		iMemberId = requestProperties.getLong(ICIConstants.ICI_PP_MEMBER_ID, null);
		iCatalogEntryId = requestProperties.getLong(ICIConstants.ICI_PP_CATALOG_ENTRY_ID, null);
		iProductSKUNumber = requestProperties.getString(ICIConstants.ICI_PP_PART_NUMBER, null);
		iCurrency = requestProperties.getString(ICIConstants.ICI_PP_CURRENCY,null);
		iPrice = requestProperties.getBigDecimal(ICIConstants.ICI_PP_LIST_PRICE,null);
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_TRANSPORT_ADAPTER, this.getClass().getName(), "setRequestPropertiesPerCommandInstance");
}
}
