//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.negotiation.commands.*;

import java.sql.*;
import java.util.*;
import java.math.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;
import java.io.*;
import java.lang.*;

import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.base.objects.*;


import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.datatype.*;



import com.ibm.commerce.negotiation.objects.*;
import com.ibm.commerce.negotiation.util.*;
import com.ibm.commerce.negotiation.misc.*;
import com.ibm.commerce.messaging.outboundservice.*;

import com.ibm.commerce.common.objects.*;

/**
 * Default implementation of  NotifyCloseAuctionCmd.
 * Notify the winners by creating new messages in their message box.
 * @author: Jianren Li, Ning Yan
 */ 
public class NotifyCompleteOrderCmdImpl extends NotifyAuctionBaseCmdImpl implements NotifyCompleteOrderCmd 
{
	private java.lang.Long bidReferenceNumber;
	private com.ibm.commerce.negotiation.objects.BidAccessBean bidAccessBean = null;
	protected java.lang.Long orderId = null;
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
/**
 * NotifyCompleteOrderCmdImpl constructor.
 */
public NotifyCompleteOrderCmdImpl() {
	super();
}
/**
 * Get an instance of BidAccessBean. 
 * @return com.ibm.commerce.negotiation.objects.BidAccessBean
 */
public com.ibm.commerce.negotiation.objects.BidAccessBean getBidAccessBean() 
   throws FinderException, RemoteException, NamingException, CreateException 
{
	
   if((bidAccessBean == null) && getBidReferenceNumber()!=null) 
   {

	  BidAccessBean aBean = new BidAccessBean();
	  aBean.setInitKey_id(getBidReferenceNumber());
	  aBean.refreshCopyHelper();
	  setBidAccessBean(aBean);
   }
   return bidAccessBean;
}
/**
 * Get a bid reference number
 * @return java.lang.Long
 */
public java.lang.Long getBidReferenceNumber() 
{
   if( bidReferenceNumber != null )
   { 
	  return bidReferenceNumber;
   }
   else if ( bidAccessBean != null )
   { 
	  try
	  {
	     return bidAccessBean.getIdInEJBType();
	  } 
	  catch(Exception e)
	  {
	     return null;
	  }
   }
   else
   {
	  return null;
   }
}
/**
 * method to get OrderId.
 * @return java.lang.Long
 */
public java.lang.Long getOrderId() {
	return orderId;
}
/**
 * This method calls Commerce Suite messaging system
 * to load jsp's and generate a message, then writes
 * the message into message table,  and adds an entry into
 * msgmemrel table for the corresponding recipients 
 * @exception com.ibm.commerce.exception.ECException.
 */
public void performExecute() throws ECException 
{
	ECTrace.entry(
		ECTraceIdentifiers.COMPONENT_NEGOTIATION, 
		this.getClass().getName(), 
		"performExecute"); 

	CommandContext cc = getCommandContext();
	String view1, view2 = null;
	Messaging msg1, msg2 = null;
	byte[] content;
	String subject, body;
	Integer storeId = null;
	TypedProperty inparms = new TypedProperty();

	parseDelayString();
	Long origuserid = getUserId();
	try
	{
		if (getBidAccessBean() == null)
		{
			throw new ECApplicationException(
				ECMessage._ERR_DIDNT_SET_BID_PARM, 
				this.getClass().getName(), 
				"performExecute"); 
		}

		if (getOrderId() == null)
		{
			throw new ECApplicationException(
				ECMessage._ERR_DIDNT_SET_ORDER, 
				this.getClass().getName(), 
				"performExecute"); 
		}

		AuctionAccessBean au = new AuctionAccessBean();
		au.setInitKey_id(getBidAccessBean().getAuctionIdInEJBType());
		au.refreshCopyHelper();
		Long memId = au.getOwnerIdInEJBType();

		//-------------------------------------------------------------------------------
		// Send message to the shopper whose order has been completed
		//-------------------------------------------------------------------------------
		//message's owner same as auction's
		MessageAccessBean mBean = new MessageAccessBean(memId);

		mBean.setMsgType(AuctionConstants.EC_MESSAGE_TYPE_ORDER_COMPLETED);

		//set bid_id into MSGPARAM1 column of message table
		mBean.setMsgParam1(getBidReferenceNumber().toString());


		//We set defaultStoreId as 0 for now because otherwise the code will not work
		//Integer defaultStoreId = new Integer("0");

		storeId = getBidAccessBean().getStoreIdInEJBType();

		//If we have a storeId from bid which different from that in CommandContext,
	    //update both storeId and storeAccessBean in CommandContext, so messaging system
	    //can get correct storeId.  This storeAccessBean will be used to append store directory.
	    StoreAccessBean sab = cc.getStore();
	    if( !storeId.equals( cc.getStoreId()) )
	    {
	       sab= cc.getStore(storeId);
	       cc.setStore(sab);
	       cc.setStoreId( storeId );
		}

		// 17=msgty

		view2 = EC_MESSAGE_COMPLETEORDER_VIEW_CMD; 
		// this is the name of a viewcmd    
		// OrderCompleteSubject
		msg2 = 
			new Messaging(new Integer(EC_MESSAGE_TYPE_COMPLETEORDER), storeId); 
		// 18=msgty

		String userId = getBidAccessBean().getOwnerId();
		Integer langId = getPreferredLanguageId(sab, getBidAccessBean().getOwnerIdInEJBType());
		inparms.put(ECConstants.EC_LANGUAGE_ID, langId.toString()); // langId

		//Pass along storeId
	    inparms.put(ECConstants.EC_STORE_ID, storeId.toString());

		inparms.put(ECConstants.EC_ORDER_RN, orderId.toString()); // orderId

		CommandContext cmdContextClone = (CommandContext)cc.clone();
		cmdContextClone.setStoreId(storeId);
		cmdContextClone.setUserId(getBidAccessBean().getOwnerIdInEJBType());
		cmdContextClone.setStore(sab);
		cmdContextClone.setLanguageId(langId);
		cmdContextClone.setStoreCommand(true);
		cmdContextClone.resetStoreRef();


		//get msg subject
		subject = AuctionShopperMessageHelper.getUserMessage(getCommandContext().getStore(), langId, "orderCompleteSubject" , null );
		mBean.setSubject(subject.trim());

		//-------
		// msg content
		//-------
		inparms.put(EC_BID_ID, getBidReferenceNumber().toString()); // bid_id
		msg2.compose(view2, cmdContextClone, inparms); 
		// comContext is the command context 
		content = msg2.getContent(new Integer(1), null); // transport
		try
	    {
	       body = new String(content, "UTF8");
	    }
	    catch(Exception e)
	    {
	       body = "";
	    }
		mBean.setContent(body.trim());

		mBean.setDelayTimePeriod(
			this.getDelayDays(), 
			this.getDelayHours(), 
			this.getDelayMins()); 

		mBean.commitCopyHelper();
		try
		{
			mBean.sendTo(getBidAccessBean().getOwnerIdInEJBType());
		}
		catch (Exception e)
		{
			//do a trace;

		}

	}
	catch (FinderException e)
	{
		throw new ECApplicationException(
			ECMessage._ERR_FINDER_EXCEPTION, 
			this.getClass().getName(), 
			"performExecute", 
			ECMessageHelper.generateMsgParms(
				AuctionConstants.EC_AUCTION_REF_NUM, 
				getAuctionReferenceNumber())); 
	}
	catch (RemoteException e)
	{
		throw new ECApplicationException(
			ECMessage._ERR_REMOTE_EXCEPTION, 
			this.getClass().getName(), 
			"performExecute", 
			ECMessageHelper.generateMsgParms(e.getMessage())); 
	}
	catch (NamingException e)
	{
		throw new ECApplicationException(
			ECMessage._ERR_NAMING_EXCEPTION, 
			this.getClass().getName(), 
			"performExecute", 
			ECMessageHelper.generateMsgParms(e.getMessage())); 
	}
	catch (CreateException e)
	{
		throw new ECApplicationException(
			ECMessage._ERR_CREATE_EXCEPTION, 
			this.getClass().getName(), 
			"performExecute", 
			ECMessageHelper.generateMsgParms(e.getMessage())); 
	}
	finally 
	{
		getCommandContext().setUserId(origuserid);
	}

	ECTrace.exit(
		ECTraceIdentifiers.COMPONENT_NEGOTIATION, 
		this.getClass().getName(), 
		"performExecute"); 

}
/** 
 * This resets the output properties to the values they had
 * prior to calling the execute method, so that the get methods
 * no longer work but the isReadyToCallExecute returns true. 
 * It provides a convenient and efficient way to reuse the same
 * command instance with changes in some input properties or in
 * the CommandTarget. 
 */
public void reset() 
{
   super.reset();
   bidReferenceNumber = null;
   bidAccessBean = null;
   auctionAccessBean = null;
   auctionReferenceNumber = null;
   orderId = null;
   delayDays = 0;
   delayHours = 0;
   delayMins = 0; 	
}
/**
 * Set an instance of BidAccessBean. 
 * @param newBidAccessBean com.ibm.commerce.negotiation.objects.BidAccessBean
 */
public void setBidAccessBean(com.ibm.commerce.negotiation.objects.BidAccessBean newBidAccessBean) {
	bidAccessBean = newBidAccessBean;
}
/**
 * Set a bid reference number
 * @param newBidReferenceNumber java.lang.Long
 */
public void setBidReferenceNumber(java.lang.Long newBidReferenceNumber) {
	bidReferenceNumber = newBidReferenceNumber;
}
/**
 * method to set OrderId.
 * @param newOrderId java.lang.Long
 */
public void setOrderId(java.lang.Long newOrderId) {
	orderId = newOrderId;
}
}
