//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.order.calculation.*;

import com.ibm.commerce.datatype.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.server.*;
import java.math.*;
import java.rmi.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.*;
/**
 * This is the default implementation of the {@link ApplyCalculationUsageCmd
 * ApplyCalculationUsage} command.
 *
 * @see ApplyCalculationUsageCmd
 */
public class ApplyCalculationUsageCmdImpl extends CalculationCmdImpl implements ApplyCalculationUsageCmd {
	public static final String COPYRIGHT		= com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	private Item[]			iItems				= null;
	private Integer[]		inTaxCategoryIds	= null;
	private Set				iAppliedItems		= null;
public ApplyCalculationUsageCmdImpl() {
	super();
}
	/**
	 * This method is called by performExecute() to invoke the
	 * CalculationCodeApply command.
	 */
	protected void callCodeApply(Group aGroup) throws ECException {
		
		final String strMethod = "callCodeApply";
		try {
			
			CalculationMethodAccessBean	abMethod	= CalculationRegistry.getInstance().getMethod(aGroup.getCode().getCalculationCodeApplyMethodIdInEJBType());
			CalculationCodeApplyCmd		cmd			= (CalculationCodeApplyCmd)createCommand(abMethod);
			
			cmd.setCode(aGroup.getCode());
			cmd.setItems(aGroup.getItems());

			if(inTaxCategoryIds == null) {
				cmd.setValues(aGroup.getValues());
			}
			else {
				for(int i = 0; i < inTaxCategoryIds.length; i++) {
					BigDecimal[] dValues = aGroup.getValues(inTaxCategoryIds[i]);
					if(dValues != null) cmd.setValues(inTaxCategoryIds[i], dValues);
				}
			}
			
			cmd.execute();
			
		}
		catch(CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		
	}
	/**
	 * This method is called by performExecute() to invoke the
	 * CalculationCodeCalculate command.
	 */
	protected void callCodeCalculate(Group aGroup) throws ECException {
		
		final String strMethod = "callCodeCalculate";
		try {
			
			CalculationMethodAccessBean abMethod	= CalculationRegistry.getInstance().getMethod(aGroup.getCode().getCalculationMethodIdInEJBType());
			CalculationCodeCalculateCmd cmd			= (CalculationCodeCalculateCmd)createCommand(abMethod);
			
			cmd.setCode(aGroup.getCode());
			cmd.setItems(aGroup.getItems());
			cmd.setTaxCategoryIds(inTaxCategoryIds);
			
			cmd.execute();
			
			if(inTaxCategoryIds == null) {
				aGroup.setValues(cmd.getValues());
			}
			else {
				for(int i = 0; i < inTaxCategoryIds.length; i++) {
					aGroup.setValues(inTaxCategoryIds[i], cmd.getValues(inTaxCategoryIds[i]));
				}
			}
			
		}
		catch(CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		
	}
	/**
	 * This method is called by performExecute() to invoke the
	 * CalculationCodeCombine command.
	 */
	protected Group[] callCodeCombine(Item[] aItems) throws ECException {
		
		CalculationMethodAccessBean	abMethod	= CalculationRegistry.getInstance().getCodeCombineMethod(commandContext.getStoreId(), getUsageId());
		if(abMethod == null) return null;
		CalculationCodeCombineCmd	cmd			= (CalculationCodeCombineCmd)createCommand(abMethod);
		
		cmd.setItems(aItems);
		cmd.setTaxCategoryIds(inTaxCategoryIds);
		
		cmd.execute();
		return cmd.getGroups();
		
	}
	public void checkAppliedItems() throws ECException {
		
		final String strMethod = "checkAppliedItems";
		
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod, "itemIds=" + CalculationHelper.getInstance().toString(iItems));
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod, "appliedItems=" + iAppliedItems);
		
		for(int i = 0; i < iItems.length; i++) {
			
			if(iAppliedItems == null || !iAppliedItems.contains(iItems[i])) {
				
				TypedProperty exceptionData = iItems[i].createExceptionData();
				
				exceptionData.put(ECConstants.EC_ERROR_CODE			, CalculationConstants.ERRCODE_USAGE_NOT_APPLIED	);
				exceptionData.put(CalculationConstants.EC_USAGE_ID	, "" + getUsageId()									);
				
				throw new ECApplicationException(ECMessage._ERR_CALCODE, getClass().getName(), strMethod, new Object[] { getUsageId().toString(), CalculationConstants.ERRCODE_USAGE_NOT_APPLIED }, CalculationConstants.CALCODE_ERRVIEW, exceptionData);
				
			}
			
		}
		
	}
	public Set getAppliedItems() { return iAppliedItems; }
	/**
	 * Returns the items.
	 */
	protected Item[] getItems() { return iItems; }
	/**
	 * Returns the tax categories.
	 */
	protected Integer[] getTaxCategoryIds() { return inTaxCategoryIds; }
	public void performExecute() throws ECException {
		
		final String strMethod = "performExecute";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod);
		
		Group[] groups = callCodeCombine(iItems);
		if(groups != null) {		
			for(int i = 0; i < groups.length; i++) {
				Group group = groups[i];
				if(!group.isCalculated()) callCodeCalculate(group);
				if(!group.isEmpty()) {
					updateAppliedItems(group);
					callCodeApply(group);
				}
			}
		}
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod);
		
	}
	public void reset() {
		
		final String strMethod = "reset";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod);
		
		iItems						= null;
		inTaxCategoryIds			= null;
		iAppliedItems				= null;
		
		super.reset();
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod);
		
	}
public void setItems(Item[] aItems) { iItems = aItems; }
public void setTaxCategoryIds(Integer[] anTaxCategoryIds) { inTaxCategoryIds = anTaxCategoryIds; }
	/**
	 * This method is called by performExecute() to update the set of items being applied to.
	 */
	protected void updateAppliedItems(Group aGroup) {
		if(iAppliedItems == null) iAppliedItems = new HashSet();
		Integer[]	nTaxCategoryIds	= (inTaxCategoryIds == null ? new Integer[] { null } : inTaxCategoryIds);
		Item[]		items			= aGroup.getItems();
		for(int i = 0; i < nTaxCategoryIds.length; i++) {
			BigDecimal[] dValues = aGroup.getValues(nTaxCategoryIds[i]);
			if(dValues == null) continue;
			for(int j = 0; j < items.length; j++) {
				if(dValues[j] != null) iAppliedItems.add(items[j]);
			}
		}
	}
}
