//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.returns.commands.*;

import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.ordermanagement.objects.*;
import java.sql.*;
import java.util.*;

/**
 * Command used to determine what means to use for a refund. This implementation chooses
 * the payment vehicle used for the most recent order with a order item on this return mechandise authorization (RMA).  Sets
 * the order number in RMA.REFUNDAGAINSTORDID.
 */

public class DetermineReturnCreditVehicleCmdImpl extends com.ibm.commerce.command.TaskCommandImpl implements DetermineReturnCreditVehicleCmd {
	private static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	private final static java.lang.String className = DetermineReturnCreditVehicleCmdImpl.class.getName();
	Vector vRMAItemAB = null;
	RMAAccessBean rmaAB = null;
/**
 * DetermineReturnCreditVehicleCmdImpl constructor.
 */
public DetermineReturnCreditVehicleCmdImpl() {
	super();
}
/**
 * Pick an order number who's payment method will be used as refund method.  The most recent
 * order is picked.
 */
 /* AW 10/04/01 Fixed Bad order data message to pass in order number and field
 */
private Long determineCreditAgainstOrderID() throws ECException {
	final String methodName = "determineCreditAgainstOrderID";

	Long OrderID = null;

	RMAItemAccessBean abRMAItem = null;

	try {
		OrderItemAccessBean abOrderItem = null;
		OrderAccessBean abOrder = null;
		Long orderItemID = null;
		Long newOrderID = null;
		Timestamp oldDate = new Timestamp(1);
		Timestamp newDate = new Timestamp(1);

		/* Loop through all the RMAitems */
		for (int i = 0; i < vRMAItemAB.size(); i++) {
			/* Get next RMA item */
			abRMAItem = (RMAItemAccessBean) vRMAItemAB.elementAt(i);

			/* Get it's corresponding Order Item -- if any */
			orderItemID = abRMAItem.getOrderItemsIdInEJBType();
			if (orderItemID == null) {
				/* This one didn't come from an order.  Skip to the next RMA item. */
				continue;
			}
			/* Get the Order Id from the OrderItems Id */
			abOrderItem = new OrderItemAccessBean();
			String temp = orderItemID.toString();
			abOrderItem.setInitKey_orderItemId(orderItemID.toString());
			abOrderItem.refreshCopyHelper();
			newOrderID = abOrderItem.getOrderIdInEJBType();
			abOrder = new OrderAccessBean();
			abOrder.setInitKey_orderId(newOrderID.toString());
			abOrder.refreshCopyHelper();
			newDate = abOrder.getPlaceOrderTimeInEJBType();
			if (newDate == null) {
				/* Should never hit this.  #1 cause is dummied order data. In the real world, a order is placed (and 
				   has a placed time) before it is shipped. */  
				throw new ECApplicationException(ECMessage._ERR_BAD_ORD_DATA,className,methodName,ECMessageHelper.generateMsgParms(newOrderID.toString(), Misc.EC_ORDER_COL_TIMEPLACED));
			}
			if (newDate.after(oldDate)) {
				OrderID = newOrderID;
				oldDate = newDate;
			}

		}
	}
	catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, className, methodName, ex);
	}
	catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, className, methodName, ex);
	}
	catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, className, methodName, ex);
	}
	catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, className, methodName, ex);
	}

	/* Throw specific error if we didn't find a credit vehicle */
	if (OrderID == null) {
		throw new ECSystemException(ECMessage._ERR_CANNOT_DETERMINE_CREDIT_VEHICLE, className, methodName);
	}
	else
		return OrderID;
}
/**
 * Execute the command.
 */
 /* AW 010814 Moved check of whether there is anything to credit from
              ReturnPrepare to here.  If there's nothing to credit
              then set credit vehicle to null.
 */
public void performExecute() throws ECException {
	final String methodName = "performExecute";
	Long orderID = null;
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);

	/* check that all parameters are supplied */
	super.performExecute();

	/* determine the order id to get the payment/credit vehicle from */
	try {
		if (rmaAB.getTotalCreditInEJBType().compareTo(Misc.BIGDECIMAL_ZERO) == 1) { //greater than 0.00
			orderID = this.determineCreditAgainstOrderID();
		}
	}
	catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, className, methodName, ex);
	}
	catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, className, methodName, ex);
	}
	catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, className, methodName, ex);
	}
	catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, className, methodName, ex);
	}
	
	/* That's it.  Set it as the credit against order id */
	rmaAB.setRefundAgainstOrdId(orderID);

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);
}
public void reset() {
	rmaAB = null;
	vRMAItemAB = null;
}
/**
 * Set the RMA being accessed.
 * @param rmaAB the RMA access bean.
 */
public void setRMAAB(RMAAccessBean rmaAB) {
	this.rmaAB = rmaAB;
}
/**
 * Set the RMA items to evaluate for credit vehicles
 * @param vRMAItemAB vector of RMA item access beans.
 */
public void setRMAItemABVector(Vector vRMAItemAB) {
	this.vRMAItemAB = vRMAItemAB;
}
public void validateParameters() throws ECException {
	final String methodName = "validateParameters";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);

	/* check the RMA */
	if (rmaAB == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,className,methodName,
																		 new Object[] { ReturnConstants.EC_RMA_ID });
	}

	/* check the enumeration of RMA items */
	if (vRMAItemAB == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,className,methodName,
																		 new Object[] { ReturnConstants.EC_RMA_ITEM_ID });
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, className, methodName);
}
}
