//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.tools.promotions.*;

import com.ibm.sfc.cmd.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.webcontroller.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.order.utils.*;
import java.sql.*;
import java.util.*;
import com.ibm.commerce.utils.*;
import java.math.*;
import com.ibm.commerce.command.TaskCommandImpl;
import com.ibm.commerce.copyright.IBMCopyright;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.exception.ECSystemException;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.naming.NamingException;
import java.rmi.RemoteException;

/**
 * This task command which implements CreateDiscountCmd interface is creating a discount
 * @author: Scott Lazaruk
 */
public class CreateDiscountCmdImpl extends TaskCommandImpl implements	CreateDiscountCmd,
																		ECPromotionsConstants
{
	public static final String COPYRIGHT = IBMCopyright.SHORT_COPYRIGHT;
	protected String code;
	protected Integer storeId;
	protected Integer langId;
	protected Integer groupBy;
	protected String description;
	protected String currency;
	protected Vector assignedShopperGroups;
	protected boolean allShoppers;
	protected boolean hasDateTimeRange;
	protected Timestamp startTimestamp;
	protected Timestamp endTimestamp;
	protected Vector rangeFromVector;
	protected Vector discountValueVector;
	protected Integer rangeType;
	protected Integer discType;
	protected Integer discSubType;
	protected Integer calCodeId;
	protected Integer calScaleId;
	protected Integer calRuleId;
	protected String adminDescription;
/**
 * Create a Discount Object.
 */
public CreateDiscountCmdImpl() {
	super();
}
/**
 * Create and populate the calCodeBean
 */
public void createCalCodeBean() throws ECSystemException {
	final String methodName= "createCalCodeBean()";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
	try
	{
		CalculationCodeAccessBean calCodeBean= 
			new CalculationCodeAccessBean(
				CalculationCodeConstants.CALCULATION_USAGE_DISCOUNT, 
				CalculationCodeConstants.DefaultDiscountCalculationMethodIds.CalculationCodeCalculateCmd, 
				CalculationCodeConstants.DefaultDiscountCalculationMethodIds.DiscountCalculationCodeApplyCmd, 
				CalculationCodeConstants.DefaultDiscountCalculationMethodIds.CalculationCodeQualifyCmd, 
				storeId,code); 

		if (discType.intValue() == PRODUCT_LEVEL)
		{
			calCodeBean.setGroupBy(groupBy);
			calCodeBean.setDisplayLevel(ORDERITEM_DISPLAY);
		}
		else if (discType.intValue() == ORDER_LEVEL)
		{
			calCodeBean.setDisplayLevel(ORDER_DISPLAY);
		}
		calCodeBean.setPublished(DISCOUNT_PUBLISHED);
		calCodeBean.setCombination(CalculationCodeConstants.IN_COMBINATION_WITH);
		// use timestamp helper to access now()
		TimestampHelper tsH= new TimestampHelper(0);
		calCodeBean.setLastUpdate(tsH.now());
		// add some new columns for new schema
		calCodeBean.setDescription(adminDescription);
		if (hasDateTimeRange)
		{
			calCodeBean.setStartDate(startTimestamp);
			calCodeBean.setEndDate(endTimestamp);
		}
		else
		{
			calCodeBean.setStartDate(tsH.now());
		}
		if (!allShoppers)
		{
			calCodeBean.setFlags(new Integer(CalculationCodeConstants.FLAGS_RULE_RESTRICTED));

		}
		
				
		calCodeBean.commitCopyHelper();
		calCodeId= calCodeBean.getCalculationCodeIdInEJBType();
	}
	catch (FinderException e)
	{
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (RemoteException e)
	{
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (CreateException e)
	{
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (NamingException e)
	{
		throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (Exception e)
	{
		com.ibm.commerce.server.TransactionManager.rollback();
		throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
/**
 * Create and populate the calDescBean
 */
public void createCalDescBean() throws ECSystemException
{
	final String methodName = "createCalDescBean()";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
	try
	{
		CalculationCodeDescriptionAccessBean calDescCodeBean = new CalculationCodeDescriptionAccessBean(calCodeId, langId, description);
		calDescCodeBean.commitCopyHelper();
	}
	catch (FinderException e)
	{
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (RemoteException e)
	{
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (CreateException e)
	{
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (NamingException e)
	{
		throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (Exception e)
	{
		com.ibm.commerce.server.TransactionManager.rollback();
		throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
/**
 * Create and populate the calRangeBean
 */
public void createCalRangeBean() throws ECSystemException {
	final String methodName = "createCalRangeBean()";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
	try
	{
		int numRanges = rangeFromVector.size();
		int i = 0;
		Integer calRangeId;
		for (i = 0; i < numRanges; i++)
		{
			Integer calMethodId;
			if (discType.intValue() == ORDER_LEVEL)
			{
				if (discSubType.intValue() == BY_PERCENTAGE)
				{
					calMethodId = CalculationCodeConstants.DefaultDiscountCalculationMethodIds.PercentageCalculationRangeCmd;
				}
				else
				{
					calMethodId = CalculationCodeConstants.DefaultDiscountCalculationMethodIds.FixedAmountCalculationRangeCmd;
				}
			}
			else
			{
				if (discSubType.intValue() == BY_PERCENTAGE)
				{
					calMethodId = CalculationCodeConstants.DefaultDiscountCalculationMethodIds.PercentageCalculationRangeCmd;
				}
				else if (discSubType.intValue() == BY_FIXED_AMOUNT)
				{
					calMethodId = CalculationCodeConstants.DefaultDiscountCalculationMethodIds.FixedAmountCalculationRangeCmd;
				}
				else
				{
					if (rangeType.intValue() == CURRENCY_RANGE)
					{
	                    calMethodId = CalculationCodeConstants.DefaultDiscountCalculationMethodIds.FixedAmountCalculationRangeCmd;
					}
					else
					{
						calMethodId = CalculationCodeConstants.DefaultDiscountCalculationMethodIds.PerUnitAmountCalculationRangeCmd;
					}
				}
			}

			CalculationRangeAccessBean calRangeBean = new CalculationRangeAccessBean(calScaleId, new BigDecimal((String) rangeFromVector.elementAt(i)), calMethodId);

			calRangeBean.setCumulative(CUMULATIVE_FALSE);

			calRangeBean.commitCopyHelper();
			calRangeId = calRangeBean.getCalculationRangeIdInEJBType();

			CalculationRangeLookupResultAccessBean calRLBean = new CalculationRangeLookupResultAccessBean(calRangeId);
			if ((discSubType.intValue() == BY_FIXED_AMOUNT) || (discSubType.intValue() == BY_AMOUNT_PER_UNIT))
			{
				calRLBean.setCurrency(currency);
			}
			try
			{
				BigDecimal dValue = new BigDecimal((String) discountValueVector.elementAt(i));
				calRLBean.setValue(dValue.negate());
			}
			catch (NumberFormatException e)
			{
				throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
			}
			calRLBean.commitCopyHelper();

		}
	}
	catch (FinderException e)
	{
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (RemoteException e)
	{
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (CreateException e)
	{
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (NamingException e)
	{
		throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (Exception e)
	{
		com.ibm.commerce.server.TransactionManager.rollback();
		throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
/**
 * Create and populate the calRuleBean
 */
public void createCalRuleBean() throws ECSystemException
{
	final String methodName = "createCalRuleBean()";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
	try
	{
		CalculationRuleAccessBean calRuleBean = new CalculationRuleAccessBean(calCodeId,CalculationCodeConstants.DefaultDiscountCalculationMethodIds.CalculationRuleCalculateCmd,CalculationCodeConstants.DefaultDiscountCalculationMethodIds.DiscountCalculationRuleQualifyCmd);
		/* ** Time fields will be add into calcode table **
		if (hasDateTimeRange)
		{
			calRuleBean.setStartDate(startTimestamp);
			calRuleBean.setEndDate(endTimestamp);
		}
		else
		{
			calRuleBean.setStartDate(TimestampHelper.now());
		}
		if (!allShoppers)
		{
			calRuleBean.setFlags(new Integer(CalculationCodeConstants.FLAGS_RULE_RESTRICTED));

		}
		*/
		calRuleBean.setCombination(CalculationCodeConstants.NOT_IN_COMBINATION_WITH);

		calRuleBean.commitCopyHelper();
		calRuleId = calRuleBean.getCalculationRuleIdInEJBType();
	}
	catch (FinderException e)
	{
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (RemoteException e)
	{
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (CreateException e)
	{
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (NamingException e)
	{
		throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (Exception e)
	{
		com.ibm.commerce.server.TransactionManager.rollback();
		throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
/**
 * Create and populate the calRuleMgpBean
 */
public void createCalRuleMgpBean() throws ECSystemException
{
	final String methodName = "createCalRuleMgpBean()";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
	int i = 0;
	Long grpRef;
	try
	{
		int numGrpsAssigned = assignedShopperGroups.size();
		for (i = 0; i < numGrpsAssigned; i++)
		{
			grpRef = new Long((String) assignedShopperGroups.elementAt(i));
			CalculationRuleMemberGroupRelationshipAccessBean calRuleMgpBean = new CalculationRuleMemberGroupRelationshipAccessBean(calRuleId, grpRef);
			calRuleMgpBean.commitCopyHelper();
		}
	}
	catch (FinderException e)
	{
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (RemoteException e)
	{
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (CreateException e)
	{
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (NamingException e)
	{
		throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (Exception e)
	{
		com.ibm.commerce.server.TransactionManager.rollback();
		throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
/**
 * Create and populate the calRuleScaleBean
 */
public void createCalRuleScaleBean() throws ECSystemException
{
	final String methodName = "createCalRuleScaleBean()";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
	try
	{
		CalculationRuleScaleRelationshipAccessBean calRuleScaleBean = new CalculationRuleScaleRelationshipAccessBean(calScaleId, calRuleId);
		calRuleScaleBean.commitCopyHelper();
	}
	catch (FinderException e)
	{
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (RemoteException e)
	{
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (CreateException e)
	{
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (NamingException e)
	{
		throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (Exception e)
	{
		com.ibm.commerce.server.TransactionManager.rollback();
		throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
/**
 * Create and populate the calScaleBean
 */
public void createCalScaleBean() throws ECSystemException {
    final String methodName = "createCalScaleBean()";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
    try {
        Integer calMethodId;
        if (rangeType.intValue() == QTY_RANGE) {
            if (discSubType.intValue() == BY_AMOUNT_PER_UNIT) {
                calMethodId = CalculationCodeConstants.DefaultDiscountCalculationMethodIds.QuantityCalculationScaleLookupCmd;
            } else {
                calMethodId =
                    CalculationCodeConstants.DefaultDiscountCalculationMethodIds.QuantitySpreadByNetPriceCalculationScaleLookupCmd;
            }

        } else {

            if (discSubType.intValue() == BY_AMOUNT_PER_UNIT) {
                // new calmethod from Alex -- patch
                calMethodId =
                    CalculationCodeConstants
                        .DefaultDiscountCalculationMethodIds
                        .NetPriceWithQuantityAsResultMultiplierCalculationScaleLookupCmd;
            } else {
                calMethodId = CalculationCodeConstants.DefaultDiscountCalculationMethodIds.NetPriceCalculationScaleLookupCmd;
            }
        }

        CalculationScaleAccessBean calScaleBean =
            new CalculationScaleAccessBean(storeId, CalculationCodeConstants.CALCULATION_USAGE_DISCOUNT, calMethodId);
        if (rangeType.intValue() == QTY_RANGE) {
            calScaleBean.setQuantityUnitId(CalculationCmdHelper.UNITLESS);
        } else {
            calScaleBean.setCurrency(currency);
        }
        calScaleBean.setCode(code);

        calScaleBean.commitCopyHelper();
        calScaleId = calScaleBean.getCalculationScaleIdInEJBType();
    } catch (FinderException e) {
        throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), methodName, e);
    } catch (RemoteException e) {
        throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), methodName, e);
    } catch (CreateException e) {
        throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), methodName, e);
    } catch (NamingException e) {
        throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), methodName, e);
    } catch (Exception e) {
        com.ibm.commerce.server.TransactionManager.rollback();
        throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
    }
    ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
/**
 * Create and populate CatalogEntryCalculationCode Bean.
 * @exception com.ibm.commerce.exception.ECSystemException If error exists during creation.
 */
public void createCatencalcd() throws com.ibm.commerce.exception.ECSystemException {
    final String methodName = "createCatencalcd()";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);

    if (discType.intValue() == ORDER_LEVEL)
    {
        try
        {
            CatalogEntryCalculationCodeAccessBean aCatCalcd = new CatalogEntryCalculationCodeAccessBean(storeId, calCodeId, null);
            aCatCalcd.commitCopyHelper();
        }
        catch (Exception e)
        {
            ECTrace.trace(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName, e.toString());
        }
    }
    ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
/**
 * Provide the property Administration Description.
 * @return java.lang.String
 */
public java.lang.String getAdminDescription() {
	return adminDescription;
}
/**
 * Provide the property assignedShopperGroups.
 * @return Vector
 */
public Vector getAssignedShopperGroups() {
	return assignedShopperGroups;
}
/**
 * Provide the property Calculation Code.
 * @return java.lang.Integer
 */
public java.lang.Integer getCalCodeId() {
	return calCodeId;
}
/**
 * Provide the property Code, A character string that uniquely identifies this CalculationCode.
 * @return java.lang.String
 */
public java.lang.String getCode() {
	return code;
}
/**
 * Provide the property Currency.
 * @return java.lang.String
 */
public java.lang.String getCurrency() {
	return currency;
}
/**
 * Provide the property Shopper's Description.
 * @return java.lang.String
 */
public java.lang.String getDescription() {
	return description;
}
/**
 * Provide the property Discount Values.
 * @return java.util.Vector
 */
public java.util.Vector getDiscountValueVector() {
	return discountValueVector;
}
/**
 * Provide the subtypes of discount.
 * @return java.lang.Integer
 */
public java.lang.Integer getDiscSubType() {
	return discSubType;
}
/**
 * Provide the property Discount Type.
 * @return java.lang.Integer
 */
public java.lang.Integer getDiscType() {
	return discType;
}
/**
 * Provide the property End Timestamp of the discount Obj.
 * @return java.sql.Timestamp
 */
public java.sql.Timestamp getEndTimestamp() {
	return endTimestamp;
}
/**
 * Provide the property GroupBy.
 * @return java.lang.Integer
 */
public java.lang.Integer getGroupBy() {
	return groupBy;
}
/**
 * Provide the property Language ID.
 * @return java.lang.Integer
 */
public java.lang.Integer getLangId() {
	return langId;
}
/**
 * Vector of the values that each Discount Range starts from.
 * @return java.util.Vector
 */
public java.util.Vector getRangeFromVector() {
	return rangeFromVector;
}
/**
 * Provide the property Discount Range type.
 * @return java.lang.Integer
 */
public java.lang.Integer getRangeType() {
	return rangeType;
}
/**
 * Provide the property Start Timestamp of the discount Obj.
 * @return java.sql.Timestamp
 */
public java.sql.Timestamp getStartTimestamp() {
	return startTimestamp;
}
// **** bb you should ensure that the parameters that you need to have set are set.  For example, check
// that rangeType is not null. ****
	public boolean isReadyToCallExecute(){return true;}
	
/**
 * Task command execution method, doing the actual work, populating the tables, creating objects.
 */
public void performExecute() throws ECSystemException, ECException
{
	final String methodName = "performExecute";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
	try
	{
		storeId = this.getCommandContext().getStoreId();
		langId = this.getCommandContext().getLanguageId();
	}
	catch (Exception e)
	{
	    com.ibm.commerce.server.TransactionManager.rollback();
		throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
	};

	createCalCodeBean();
	createCalDescBean();
	createCalRuleBean();
	if (!allShoppers)
	{
//		createCalRuleMgpBean();
		createCalCodeMgpBean();
	}
	createCalScaleBean();
	createCalRuleScaleBean();
	createCatencalcd();
	createCalRangeBean();

	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
/**
 * set the property Administration Description.
 * @param newDescription java.lang.String
 */
public void setAdminDescription(java.lang.String newAdminDescription) {
	adminDescription = newAdminDescription;
}
/**
 * set the property allShoppers, if the discount is for all shoppers, this will be set to true.
 * @param newAllShoppers boolean
 */
public void setAllShoppers(boolean newAllShoppers) {
	allShoppers = newAllShoppers;
}
/**
 * set the property assignedShopperGroups
 * @param newAssignedShopperGroups Vector
 */
public void setAssignedShopperGroups(Vector newAssignedShopperGroups) {
	assignedShopperGroups = newAssignedShopperGroups;
}
/**
 * Set the property Calculation Code ID.
 * @param newCalCodeId java.lang.Integer
 */
public void setCalCodeId(java.lang.Integer newCalCodeId) {
	calCodeId = newCalCodeId;
}
/**
 * Set the property code.
 * @param newCode java.lang.String
 */
public void setCode(java.lang.String newCode) {
	code = newCode;
}
/**
 * Set the property Currency.
 * @param newCurrency java.lang.String
 */
public void setCurrency(java.lang.String newCurrency) {
	currency = newCurrency;
}
/**
 * Set the property Shopper's Description.
 * @param newDescription java.lang.String
 */
public void setDescription(java.lang.String newDescription) {
	description = newDescription;
}
/**
 * Set the property DiscountValueVector.
 * @param newDiscountValueVector java.util.Vector
 */
public void setDiscountValueVector(java.util.Vector newDiscountValueVector) {
	discountValueVector = newDiscountValueVector;
}
/**
 * Set the property Discount Sub-Type.
 * @param newDiscSubType java.lang.Integer
 */
public void setDiscSubType(java.lang.Integer newDiscSubType) {
	discSubType = newDiscSubType;
}
/**
 * Set the property Discount Type.
 * @param newDiscType java.lang.Integer
 */
public void setDiscType(java.lang.Integer newDiscType) {
	discType = newDiscType;
}
/**
 * Set the property EndTimestamp.
 * @param newEndTimestamp java.sql.Timestamp
 */
public void setEndTimestamp(java.sql.Timestamp newEndTimestamp) {
	endTimestamp = newEndTimestamp;
}
/**
 * Set the property GroupBy.
 * @param newGroupBy java.lang.Integer
 */
public void setGroupBy(java.lang.Integer newGroupBy) {
	groupBy = newGroupBy;
}
/**
 * Set the property hasDateTimeRange, this will be true if the Discount Obj has the Time Limit or Schedule.
 * @param new_hasDateTimeRange boolean
 */
public void setHasDateTimeRange(boolean newHasDateTimeRange) {
	hasDateTimeRange = newHasDateTimeRange;
}
/**
 * Set the property LanguageId, this method will be removed in release.
 * @param newLangId java.lang.Integer
 */
// **** bb this method should be removed since you are getting the lang id from the command context ****
public void setLangId(java.lang.Integer newLangId) {
	langId = newLangId;
}
/**
 * Set the property RangeFromVector.
 * @param newRangeFromVector java.util.Vector
 */
public void setRangeFromVector(java.util.Vector newRangeFromVector) {
	rangeFromVector = newRangeFromVector;
}
/**
 * Set the property Range Type.
 * @param newRangeType java.lang.Integer
 */
public void setRangeType(java.lang.Integer newRangeType) {
	rangeType = newRangeType;
}
/**
 * Set the property startTimestamp.
 * @param newStartTimestamp java.sql.Timestamp
 */
public void setStartTimestamp(java.sql.Timestamp newStartTimestamp) {
	startTimestamp = newStartTimestamp;
}
/**
 * Set the property storeId, this method will be removed in next release.
 * @param newStoreId java.lang.Integer
 */

// **** bb this method should be removed since you are getting the store from the command context ****
public void setStoreId(java.lang.Integer newStoreId) {
	storeId = newStoreId;
}
// **** bb I'd move this to a separate util class ****
private int toInt(String value)
{
	try
	{
		return Integer.parseInt(value);
	}
	catch (NumberFormatException e)
	{
	}
	return 0;
}

/**
 * Create and populate calcodeMgp Bean
 */
public void createCalCodeMgpBean() throws ECSystemException
{
	final String methodName = "createCalCodeMgpBean()";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
	int i = 0;
	Long grpRef;
	try
	{
		int numGrpsAssigned = assignedShopperGroups.size();
		for (i = 0; i < numGrpsAssigned; i++)
		{
			grpRef = new Long((String) assignedShopperGroups.elementAt(i));
//			CalculationRuleMemberGroupRelationshipAccessBean calRuleMgpBean = new CalculationRuleMemberGroupRelationshipAccessBean(calRuleId, grpRef);
			CalculationCodeMemberGroupAccessBean calCodeMgpBean= new CalculationCodeMemberGroupAccessBean(calCodeId,grpRef);
			calCodeMgpBean.commitCopyHelper();
		}
	}
	catch (FinderException e)
	{
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (RemoteException e)
	{
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (CreateException e)
	{
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (NamingException e)
	{
		throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), methodName, e);
	}
	catch (Exception e)
	{
		com.ibm.commerce.server.TransactionManager.rollback();
		throw new ECSystemException(ECMessage._ERR_GENERIC, getClass().getName(), methodName, e);
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_MERCHANDISING, getClass().getName(), methodName);
}
}